#!/bin/pfsh -- # perl, to stop looping
eval 'exec /usr/perl5/bin/perl -w -S $0 ${1+"$@"}'
  if 0;
#
# Copyright 2001-2002 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#ident	"@(#)apache-script.pl	1.12	02/02/20 SMI"
#
# Set up apache instance

use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Sun::Solaris::Utils qw(gettext);
use Cluster::Cgi;
use Cluster::Common;

(new Cluster::Common)->run_script(\&run_script);

sub run_script {
    my ($q, $outfd) = @_;

    # Validate input
    my $i;
    foreach $i ('confhost', 'codetype', 'doctype', 'adir',
	    'docdir', 'cgidir', 'logdir', 'server', 'bindaddr', 'port') {
	($q->param($i) ne "") || die(sprintf(gettext("Parameter %s undefined"), $i));
    }

    my $hostname = `hostname`;
    chop $hostname;

    if (defined $q->param("pkgcd")) {
	&addpkgs($q, $outfd);
    }

    &validate();

    # Logs are local, so create on all nodes
    &mkdirp($q, $q->param('logdir'));

    if ($q->param('codetype') eq "local" ||
	    $q->param('confhost') eq $hostname) {
	# Either we are doing local installation, so every node should
	# set this up, or we are the designated host
	&mkdirp($q, $q->param('adir'));
	&mkdirp($q, $q->param('adir')."/conf");
	print $outfd sprintf(gettext("Made directories %s and %s\n"),
	    $q->param('adir'), $q->param('logdir'));
	&setupcode($q, $outfd);
	&editfile($q, $outfd, "conf/httpd.conf");
	&editfile($q, $outfd, "bin/apachectl");
    }

    if ($q->param('doctype') eq "local" ||
	    $q->param('confhost') eq $hostname) {
	# Either we are doing local installation, so every node should
	# set this up, or we are the designated host
	&mkdirp($q, $q->param('docdir'));
	&mkdirp($q, $q->param('cgidir'));
	print $outfd sprintf(gettext("Made directories %s and %s\n"),
	    $q->param('docdir'), $q->param('cgidir'));
	setupdocs($q, $outfd);
    }

    &disablerc($outfd);
}

# Make a directory
sub mkdirp {
    my ($q, $dir) = @_;
    my $status = system("mkdir -p $dir");
    if ($status) {
	die($q->sprintfn(gettext("mkdir %1 failed: %2"), $dir, $?));
    }
    chown((getpwnam("nobody"))[2,3], $dir);
}

# Validate the packages
sub validate {
    if (! -d "/usr/apache/bin") {
	die(gettext("SUNWapchu not installed"));
    }
    if (! -f "/etc/apache/httpd.conf-example") {
	die(gettext("SUNWapchr not installed"));
    }
}

# Set up the binaries and conf files
sub setupcode {
    my ($q, $outfd) = @_;
    my ($status, $cmd);

    if ($q->param('adir') ne "/usr/apache") {
	my ($subdir);
	foreach $subdir ("bin", "man", "libexec") {
	    # man not required
	    if ($subdir eq "man" && ! -d "/usr/apache/$subdir") {
		next;
	    }
	    $cmd = "cp -r /usr/apache/$subdir ".$q->param('adir');
	    print $outfd "$cmd\n";
	    $status = system($cmd);
	    if ($status) {
		die(sprintf(gettext("%s failed"), $cmd));
	    }
	}
    }

    $cmd = "cp -r /etc/apache/* ".$q->param('adir')."/conf"; # */ bug 4398335
    $status = system($cmd);
    print $outfd "$cmd\n";
    if ($status) {
	die(sprintf(gettext("%s failed"), $cmd));
    }
    $cmd = "cp /etc/apache/httpd.conf-example ".$q->param('adir').
	"/conf/httpd.conf";
    $status = system($cmd);
    print $outfd "$cmd\n";
    if ($status) {
	die(sprintf(gettext("%s failed"), $cmd));
    }
}

# Set up the docs dir by copying from htdocs
sub setupdocs {
    my ($q, $outfd) = @_;
    my ($status, $cmd);
    if ($q->param('docdir') ne "/usr/apache/htdocs" &&
	    -d "/usr/apache/htdocs") {
	$cmd = "cp -r /usr/apache/htdocs/manual ".$q->param('docdir');
	$status = system($cmd);
	print $outfd "$cmd\n";
	if ($status) {
	    die(sprintf(gettext("%s failed"), $cmd));
	}

	$cmd = "cp -r /var/apache/htdocs/* ".$q->param('docdir'); # */ #4398335
	print $outfd "$cmd\n";
	$status = system($cmd);
	if ($status) {
	    die(sprintf(gettext("%s failed"), $cmd));
	}
    }
}

# Disable the rc script
sub disablerc {
    my ($outfd) = @_;
    my ($cmd, $status);
    if (-e "/etc/rc3.d/S50apache") {
	$cmd = "mv /etc/rc3.d/S50apache /etc/rc3.d/xxS50apache";
	print $outfd "$cmd\n";
	$status = system($cmd);
	if ($status) {
	    die(sprintf(gettext("%s failed"), $cmd));
	}
    }
}

# Edit a config file
sub editfile {
    my ($q, $outfd, $filename) = @_;
    $filename = $q->param('adir')."/$filename";
    my ($dev, $ino, $mode, $nlink, $uid, $gid) = stat($filename);
    open(IN, "<$filename") || die ("can't open $filename");
    open(OUT, ">$filename.new") || die ("can't open $filename.new");
    while (<IN>) {
	if (/^#LockFile/i || /^#ServerName/i) {
	    $_ =~ s/#//;
	    # Will then get substituted below
	}
	if (/^#?BindAddress/i) {
	    $_ = "BindAddress ".$q->param('bindaddr')."\n";
	} elsif (/^#/) {
	} elsif (/^ServerRoot/i) {
	    $_ = "ServerRoot ".$q->param('adir')."\n";
	} elsif (/^Port/i) {
	    $_ = "Port ".$q->param('port')."\n";
	} elsif (/^ServerName/i) {
	    $_ = "ServerName ".$q->param('server')."\n";
	} elsif (/^DocumentRoot/i) {
	    $_ = "DocumentRoot \"".$q->param('docdir')."\"\n";
	} elsif (/HTTPD=/) {
	    $_ = "HTTPD=\"".$q->param('adir')."/bin/httpd -f ".
		$q->param('adir')."/conf/httpd.conf\"\n";
	} elsif (/\/(var|usr)\/apache\/htdocs/) {
	    $_ =~ s/\/(var|usr)\/apache\/htdocs/$q->param('docdir')/e;
	} elsif (/\/(var|usr)\/apache\/cgi-bin/) {
	    $_ =~ s/\/(var|usr)\/apache\/cgi-bin/$q->param('cgidir')/e;
	} elsif (/\/(var|usr)\/run/) {
	    $_ =~ s/\/(var|usr)\/run/$q->param('logdir')/e;
	} elsif (/\/(var|usr)\/apache\/logs/) {
	    $_ =~ s/\/(var|usr)\/apache\/logs/$q->param('logdir')/e;
	} elsif (/\/(var|usr)\/apache/) {
	    $_ =~ s/\/(var|usr)\/apache/$q->param('adir')/e;
	}
	print OUT $_;
    }
    close(IN);
    close(OUT);
    rename $filename, "$filename.old";
    rename "$filename.new", $filename;
    # Set ownership and mode back
    chown $uid, $gid, $filename;
    chmod $mode, $filename;
    print $outfd sprintf(gettext("Modified %s\n"), $filename);
}

# Add Apache packages
sub addpkgs {
    my ($q, $outfd) = @_;

    my ($status, $i);
    my ($pkglist) = "";
    foreach $i ("SUNWapchr", "SUNWapchu") {
	$status = system("/usr/bin/pkginfo -q $i");
	if ($status) {
		$pkglist .= " $i";
	}
    }
    if ($pkglist) {
	delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};
	if ((stat($q->param('pkgcd')))[4] != 0) {
	    die(gettext("Installation CD must be owned by root"));
	}
	my ($cmd) = "/usr/sbin/pkgadd -d ".$q->param('pkgcd')." -a ".
	    "/opt/SUNWscvw/cgi-bin/installation/sds/noask_pkgadd $pkglist";
	print $outfd "$cmd\n";
	$status = `$cmd 2>&1`;
	if ($?) {
	    die(sprintf(gettext("pkgadd of Apache failed %s\n"), $status));
	}
    }
}
