#!/usr/perl5/bin/perl -w
#
# Copyright 2001-2002 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
# ident	"@(#)installation.pl	1.14	02/02/21 SMI"
#
# SDS Install Step 2. 
#	Invoke installation.ksh. 
#	Comment out the /sds entry in /etc/vfstab, if necessary.
#	Add root to the sysadmin group as recommended in the SDS docs.

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Common;

# Create the html query object
my $q = new Cluster::Cgi;

# Print the header
print $q->header();

# Fork so that we can exit to the parent immediately
if (fork()) {
    exit;
}

# Remove the BASEDIR environment variables
delete @ENV{'BASEDIR', 'CLIENT_BASEDIR'};

# Create a cluster object for logging
my $cluster = new Cluster::Common;

# Get the solaris cdrom path to give to installation.ksh
my $solaris_cdrom_path = $q->param('solaris_cdrom_path');

# Create the string to call installation.ksh with
my $installation = "/opt/SUNWscvw/cgi-bin/installation/sds/installation.ksh";
if (defined $solaris_cdrom_path) {
	$installation .= " $solaris_cdrom_path";
}

# Call installation.ksh. We don't need pfexec here, since
# installation.ksh runs under pfksh.
`$installation`;

# Get return
my $ret = $?;

# Print result
if ($ret == 0) {
    $cluster->syslog("SDS Installation SUCCEEDED!");
} else {
    $cluster->syslog("SDS Installation FAILED!");
}

# Comment out the /sds mount point in /etc/vfstab
my $vfstab = "/etc/vfstab";
my $tmp = "/tmp/vfstab.$$";
open(VFSTAB, "$vfstab");
open(TMP, ">$tmp");
while (my $line = <VFSTAB>) {
    if ($line =~ /\/sds(\s+)/) {
        print TMP "#$line";
    } else {
        print TMP $line;
    }
}
close(TMP);
close(VFSTAB);
`/usr/bin/pfexec /usr/bin/mv $tmp $vfstab`;
if ($? != 0) {
    $cluster->syslog("mv $tmp $vfstab FAILED!");
}

# Touch the file that indicates we're done
my $logdir = $cluster->get_logdir();
`touch $logdir/done_sds`;

if ($? != 0) {
    $cluster->syslog("touch $logdir/done_sds FAILED!");
}

# Add root to the sysadmin group
`/usr/bin/pfexec /usr/sbin/usermod -G sysadmin root`;

if ($? != 0) {
    $cluster->syslog("usermod -G sysadmin root FAILED!");
}

# Exit completely
CORE::exit(0);
