#!/usr/perl5/bin/perl -w
#
# Copyright 2001-2002 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
# ident	"@(#)step5.pl	1.26	02/02/21 SMI"
#
# Installation page 5

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Install;
use Sun::Solaris::Utils qw(gettext);

# Set constants
use constant TRUE          => 0;
use constant FALSE         => 1;

# Create the html query object
my $q = new Cluster::Cgi;

# Create a install object
my $install = new Cluster::Install;

# Verify that the cdrom and patch paths are accessible from all nodes
my (@errors);

foreach my $node ($install->get_nodelist($q)) {
    my @result = $install->verify_paths($node, $q);
    
    if ($#result > -1) {
	push @errors, @result;
    }
}

# If we found errors, print them and exit
if ($#errors > -1) {

    # Start the html document
    $install->header($q, [gettext("Sun Cluster Installation"),
	"/cgi-bin/installation/step0.pl",
	gettext("Paths (Step 6 of 7)")]);
    $install->start_message_table($q, "/images/error_32.gif",
	gettext("Error"), gettext("Incorrect Path Information"),
	gettext("The following installation paths could not be reached:"));
    
    # Print unreachable nodes

    foreach my $error (@errors) {
	print $q->start_b(), $error, $q->end_b(), $q->br();
    }

    $install->end_message_table($q);
    print $q->p({class => "action-window-normal-text"});
    print gettext("Make sure that the specified paths are correct, and ".
	"that they can reachable by all nodes of the cluster.  Paths of ".
	"CDROM images should indicate the directory which contains the ".
	".cdtoc file.  Please return to the previous page and enter the ".
	"correct paths.");

    print $q->start_form();

    $install->start_button_table($q);

    print $q->button({ value   => gettext("  < Back  "),
		       onClick => "history.back()"
		       });
    $install->end_button_table($q, "cancel");

    print $q->end_form();
    
    # End the html document
    $install->footer($q);

    # Exit gracefully
    exit(0);
}

# Start the html document
$install->header($q, [gettext("Sun Cluster Installation"),
    "/cgi-bin/installation/step0.pl",
    gettext("Confirm Information (Step 7 of 7)")]);
print $q->p({class => "action-window-normal-text"});

# Get global values
my $cluster_name = $q->param('cluster_name');

# Print general info
print gettext("Verify that the information that you have entered is " .
    "correct. If any changes need to be made, use the Back button to return " .
    "to previous pages. When you are finished, click Begin Installation to " .
    "install the cluster.");

# Print reminder to install patches, if user is manually adding patches.
if ( ! $q->param('patch_path') ) {
    print gettext("<br><br><b>Reminder:</b> You have chosen to manually add ".
	"all the needed patches.  Please consult the appropriate patch ".
	"documentation for more detailed instructions.  When you are finished ".
	"with patch installation, then click Begin Installation.");
}

# Get the last node of the cluster
my $clustersize = $q->param('cluster_size');
my $last_node = $q->param("node_$clustersize.name");
if ($install->get_hostdomain()) {
    $last_node .= "." . $install->get_hostdomain();
}
my $port = $install->get_http_port();

# Print the form
my $action = "https://$last_node:$port/cgi-bin/installation/step6.pl";
print $q->start_form({ action => $action });
print $q->start_table({ BORDER => 0,
			CELLPADDING => 1,
			CELLSPACING => 0,
			CLASS => "action-table"});

# Print the cluster name
print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
print $q->start_td({ NOWRAP => undef});
print $q->CGI::div({CLASS => "action-window-label-text"},
    gettext("Cluster Name: "));

print $q->end_td();
print $q->start_td();

print $q->CGI::div({CLASS => "action-window-table-normal-text"},
    $q->param("cluster_name"));

print $q->end_td();
print $q->end_Tr();

# Print the cluster size
print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";

print $q->start_td({ NOWRAP => undef});
print $q->CGI::div({CLASS => "action-window-label-text"},
    gettext("Number of Nodes: "));

print $q->end_td();
print $q->start_td();

print $q->CGI::div({CLASS => "action-window-table-normal-text"},
    $q->param("cluster_size"));

print $q->end_td();
print $q->end_Tr();

# Print the Sun Cluster cdrom path
print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
print $q->start_td({ NOWRAP => undef});
print $q->CGI::div({CLASS => "action-window-label-text"},
    gettext("Sun Cluster Path: "));

print $q->end_td();
print $q->start_td();

print $q->CGI::div({CLASS => "action-window-table-normal-text"},
    $q->param("cdrom_path"));

print $q->end_td();
print $q->end_Tr();

# Print the Sun Cluster Data Services cdrom path, if needed.
if ($q->param("install_nfs") eq 'yes' || $q->param("install_apache") eq 'yes') {
	print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
	print $q->start_td({ NOWRAP => undef});
	print $q->CGI::div({CLASS => "action-window-label-text"},
	    gettext("Sun Cluster Data Services Path: "));

	print $q->end_td();
	print $q->start_td();

	print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	    $q->param("ds_cdrom_path"));

	print $q->end_td();
	print $q->end_Tr();
}

# Print the patch path
if (defined $q->param("patch_path") && $q->param("patch_path") ne "") {
	print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
	print $q->start_td({ NOWRAP => undef});
	print $q->CGI::div({CLASS => "action-window-label-text"},
			   gettext("Patch Path: "));
	
	print $q->end_td();
	print $q->start_td();
	
	print $q->CGI::div({CLASS => "action-window-table-normal-text"},
			   $q->param("patch_path"));
	
	print $q->end_td();
	print $q->end_Tr();
}

print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
print $q->start_td({ colspan => 3 });
print '&nbsp;';
print $q->end_td();
print $q->end_Tr();

# Print the SDS Installation
print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
print $q->start_td({ NOWRAP => undef});

if ($install->is_solaris8() == TRUE) {
	print $q->CGI::div({CLASS => "action-window-label-text"},
			   gettext("Install Solstice DiskSuite: "));
} else {
	print $q->CGI::div({CLASS => "action-window-label-text"},
			   gettext("Configure mirrored disksets: "));
}

print $q->end_td();
print $q->start_td();

print $q->CGI::div({CLASS => "action-window-table-normal-text"},
    ($q->param("install_sds") eq "yes" ? gettext("Yes") : gettext("No")));

print $q->end_td();
print $q->end_Tr();

# Print only if we're installing SDS
if ($q->param("install_sds") eq "yes") {

	# Print the Solaris cdrom path for Solaris 8
	if ($install->is_solaris8() == TRUE) {
		print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
		print $q->start_td({ NOWRAP => undef});
		print $q->CGI::div({CLASS => "action-window-label-text"},
				   gettext("Solaris 8 Path: "));
    
		print $q->end_td();
		print $q->start_td();
		
		print $q->CGI::div({CLASS => "action-window-table-normal-text"},
				   $q->param("solaris_cdrom_path"));
		
		print $q->end_td();
		print $q->end_Tr();
		
		print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
		print $q->start_td({ colspan => 3 });
		print '&nbsp;';
		print $q->end_td();
		print $q->end_Tr();
	}

    # Print the Apache Installation
    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ NOWRAP => undef});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	gettext("Install Apache: "));
    
    print $q->end_td();
    print $q->start_td();
    
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	($q->param("install_apache") eq "yes" ? gettext("Yes") :
	gettext("No")));
    
    print $q->end_td();
    print $q->end_Tr();
    
    if ($q->param("install_apache") eq "yes") {

	# Print the Apache Address
	print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
	print $q->start_td({ NOWRAP => undef});
	print $q->CGI::div({CLASS => "action-window-label-text"},
	    gettext("Shared Address: "));
	
	print $q->end_td();
	print $q->start_td();
	
	print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	    $q->param("apache_address"));
	
	print $q->end_td();
	print $q->end_Tr();
    }

    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ colspan => 3 });
    print '&nbsp;';
    print $q->end_td();
    print $q->end_Tr();

    # Print the NFS Installation
    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ NOWRAP => undef});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	gettext("Install NFS: "));
    
    print $q->end_td();
    print $q->start_td();
    
    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	($q->param("install_nfs") eq "yes" ? gettext("Yes") : gettext("No")));
    
    print $q->end_td();
    print $q->end_Tr();
    
    if ($q->param("install_nfs") eq "yes") {
	# Print the NFS Address
	print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
	print $q->start_td({ NOWRAP => undef});
	print $q->CGI::div({CLASS => "action-window-label-text"},
	    gettext("Logical Hostname: "));
	
	print $q->end_td();
	print $q->start_td();
	
	print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	    $q->param("nfs_address"));
	
	print $q->end_td();
	print $q->end_Tr();
    }
}

# Print out the name & transport selectors for each node
for (my $i=1; $i<=$q->param('cluster_size'); $i++) {
    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ colspan => 3 });
    print '&nbsp;';
    print $q->end_td();
    print $q->end_Tr();

    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ NOWRAP => undef});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	sprintf(gettext("Node %s Name: "), $i));


    print $q->end_td();
    print $q->start_td();

    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	$q->param("node_$i.name"));

    print $q->end_td();
    print $q->end_Tr();

    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ NOWRAP => undef});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	gettext("Adapter 1"));

    print $q->end_td();
    print $q->start_td();

    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	$q->param("node_$i.transport_1"));

    print $q->end_td();
    print $q->end_Tr();

    print $q->start_Tr(), $q->td({NOWRAP => undef}, '&nbsp;'), "\n";
    print $q->start_td({ NOWRAP => undef});
    print $q->CGI::div({CLASS => "action-window-label-text"},
	gettext("Adapter 2"));

    print $q->end_td();
    print $q->start_td();

    print $q->CGI::div({CLASS => "action-window-table-normal-text"},
	$q->param("node_$i.transport_2"));

    print $q->end_td();
    print $q->end_Tr();
}

# End the table
print $q->end_table(); 

# Print the hidden cluster fields
$q->print_hidden_cluster_fields();

$install->start_button_table($q);

print $q->button({ value   => gettext("  < Back  "),
		   onClick => "history.back()"
		   });


# Put spaces between the buttons
print '&nbsp;';

print $q->submit({ name  => 'submit_step_1',
		   value => gettext("  Begin Installation  ")
		   });
$install->end_button_table($q, "cancel");

# End the form
print $q->end_form();

# End the html document
$install->footer($q);
