#!/usr/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)status.pl	1.22	01/07/27 SMI"
#
# Node status page
#

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Node;
use Cluster::Transport;
use Sun::Solaris::Utils qw(gettext);

# Create the html query object
my $q = new Cluster::Cgi;

# Create a status object
my $status = new Cluster::Node;

# Create a transport object - to get node properties
my $transport = new Cluster::Transport;

# Check if in cluster mode
if ($status->is_clustermode() != 0) {
	# not in cluster mode
	$status->print_nonclustermode_error($q);
	exit;
}

#
# Get the parameter for this page.
# type - what type of component to display (Cluster or Node)
# name - the name of this component
#
my $type = $q->param('type');
my $name = $q->param('name');

if (!defined $type) {
    $type = "Cluster";
}

# Print the menu contents
if ($type eq "Cluster") {
    my $first_run = '/opt/SUNWscvw/logs/.first_run';

    # Print standard cluster info
    $status->header($q, gettext("Cluster Status - Click on a node to login or view more information"), "c");
    $status->print_status($q);
	
    # Print installation info
    if (-f $first_run) {

	# Print the installation information table
	print $q->start_table({ width => '100%',
				bgcolor => '#dddddd' });
	print $q->start_Tr({ align => 'left' });
	print $q->start_td();
	
	print $q->start_b();
	print gettext("Configuration:");
	print $q->end_b();

	print $q->end_td();
	print $q->end_Tr();
	print $q->end_table();
	
	print $q->end_center();

	# Get data
	my $clustername = $status->get_clustername();

	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	    $atime,$mtime,$ctime,$blksize,$blocks)
	    = stat($first_run);

	my $localtime = localtime($ctime);
	my $apache_installed = 0;
	my $nfs_installed = 0;
	my $mirror_installed = 0;
	my $stripe_installed = 0;
	my $concat_installed = 0;
	
	my ($apache_address, $nfs_address);
    
        if (-f '/opt/SUNWscvw/logs/shared_address') {
	    $apache_installed = 1;

	    open(FILE, '/opt/SUNWscvw/logs/shared_address');
	    my $line = <FILE>;
	    close(FILE);
	    chomp $line;
	    $apache_address = $line;
	}

        if (-f '/opt/SUNWscvw/logs/logical_host') {
	    $nfs_installed = 1;

	    open(FILE, '/opt/SUNWscvw/logs/logical_host');
	    my $line = <FILE>;
	    close(FILE);
	    chomp $line;
	    $nfs_address = $line;
	}

	# Look for mirror-1
	open(MOUNT, '/usr/sbin/mount |');
	while (my $line = <MOUNT>) {
	    if ($line =~ /\/global\/mirror-1/) {
		$mirror_installed = 1;
	    } elsif ($line =~ /\/global\/stripe-1/) {
		$stripe_installed = 1;
	    } elsif ($line =~ /\/global\/concat-1/) {
		$concat_installed = 1;
	    }
	}
	close(MOUNT);

	# Open the config file to write to
	open(CONFIG, '>>/opt/SUNWscvw/logs/configuration');
	
	if ($apache_installed == 1 || $nfs_installed == 1  || $mirror_installed == 1) {
	    print "<BR>&nbsp;&nbsp;";
	    print $q->sprintfn(gettext("Cluster <B>%1</B> was " .
		"installed on<B>%2</B> with the following services:"),
		$clustername, $localtime);

	    print CONFIG $q->sprintfn(gettext("Cluster <B>%1</B> was " .
		"installed on <B>%2</B> with the following services:"),
		$clustername, $localtime) . "\n\n";

	} else {
	    print "<BR>&nbsp;&nbsp;";
	    print $q->sprintfn(gettext("Cluster <B>%1</B> was " .
		"installed on<B>%2</B>."),
		$clustername, $localtime);

	    print CONFIG $q->sprintfn(gettext("Cluster <B>%1</B> was " .
		"installed on<B>%2</B>."),
		$clustername, $localtime) . "\n\n";
	}
	
	print $q->start_ul();

	if ($apache_installed == 1) {

#	    print $q->li();
#	    print "<B>Scalable Apache</B>, serving documents from ";
#	    print "<B>/global/mirror-1/apache-data</B>, and accessible through: ";
#	    print $q->start_a({ 
#		href => "http://".$apache_address,
#		target => '_top'
#	    });
#	    print "<B>http://$apache_address</B>";
#	    print $q->end_a();


	    print "<li>".sprintf(gettext("<B>Scalable Apache</B>, serving " .
		"documents from <B>/global/mirror-1/apache-data</B>, and " .
		"accessible through: <a target=\"_top\" " .
		"href=\"http://%s\"><B>http://%s</B></a>\n"),
		$apache_address, $apache_address);
	    print CONFIG sprintf(gettext("Scalable Apache, serving " .
		" documents from /global/mirror-1/apache-data, and " .
		" accessible through: http://%s\n\n"), $apache_address);
	}

	if ($nfs_installed == 1) {
	    print $q->li();
	    print sprintf(gettext("<B>Failover NFS</B>, serving files from " .
		"<B>/global/mirror-1</B>,  available to be mounted from the " .
		"logical address: <B>%s</B>"), $nfs_address);
	    print CONFIG sprintf(gettext("Failover NFS, serving files from " .
		"/global/mirror-1, available to be mounted from the " .
		"logical address: %s\n\n"), $nfs_address);
	}

	if ($mirror_installed == 1) {
	    print $q->li();
	    print gettext("<B>Cluster Filesystem</B>, online and mounted at: <B>/global/mirror-1</B>");
	    print CONFIG gettext("Cluster Filesystem, online and mounted at: /global/mirror-1\n\n");
	}

	if ($stripe_installed == 1) {
	    print $q->li();
	    print gettext("<B>Cluster Filesystem</B>, online and mounted at: <B>/global/stripe-1</B>");
	    print CONFIG gettext("Cluster Filesystem, online and mounted at: /global/stripe-1\n\n");
	}

	if ($concat_installed == 1) {
	    print $q->li();
	    print gettext("<B>Cluster Filesystem</B>, online and mounted at: <B>/global/concat-1</B>");
	    print CONFIG gettext("Cluster Filesystem, online and mounted at: /global/concat-1\n\n");
	}

	print $q->end_ul();

	# Print the installation information table
	print $q->start_table({ width => '100%' });
	print $q->start_Tr({ align => 'left' });
	print $q->start_td();
	
	print gettext("This information has been provided to show the " .
	    "cluster configuration immediately after the initial " .
	    "installation.");
	print gettext("For future reference, it has been saved to the " .
	    "file <B>/opt/SUNWscvw/logs/configuration</B> on this node.");

	print $q->end_td();
	print $q->end_Tr();
	print $q->end_table();

	print $q->center();

	# Close the config file
	close(CONFIG);

	# Remove the first run file
	if (unlink($first_run) != 1) {
	    $status->syslog("unlink($first_run) FAILED!");
	}
    }

} elsif ($type eq "Node") {
	# Start the html document
	$status->header($q, sprintf(gettext("Node %s Information"), $name),
	    "n-$name");

	# Get output of usefull commands into tmp files
	$transport->get_commandoutput();
	# print cluster-wide property
	my @node_property = $transport->get_propertylist($type, $name);
	if (lc($node_property[0][1]) eq "online") {
		# Get node's hardware configuration
		my $location = "/cgi-bin/node/get_hardware.pl";
		my @output = $status->get_http($name, $status->get_http_port(), $location);
		my $currow = $#node_property + 1;
		for (my $i = 0; $i <= $#output; $i = $i + 2) {
			$node_property[$currow][0] = $output[$i];
			$node_property[$currow][1] = $output[$i+1];
			$currow++;
		}
	}

	# Print the surrounding table
	print $q->start_table({ 
	    #bgcolor => '#dddddd',
	    align => 'center',
	    cellspacing => 0,
	    cellpadding => 5
	    });
	print $q->start_Tr();

	print $q->start_td({ valign => 'top' });
	print $q->img({ src => '/images/node_32.gif',
			valign => 'top'
			});
	print $q->end_td();

	print $q->start_td();

	# print node-specific property
	$status->print_property($q, \@node_property);

	# print login link
	print $q->start_a({ href => "telnet://$name" });
	print sprintf(gettext("Click here to telnet to node %s"), $name);
	print $q->end_a();

	print $q->end_td();
	print $q->end_Tr();

	print $q->end_table();

	# cleanup tmp command output
	$transport->cleanup();
} else {
	$status->print_error($q, sprintf(gettext("Unknown type - %s to display!"), $type));
}

# End the html document
print $q->end_html();

