#!/usr/bin/perl -w
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#pragma ident   "@(#)adapter.pl 1.5     01/11/28 SMI"
#
# Used by java applet
#
# Include needed libraries

require 5.005;
use strict;
use lib '/opt/SUNWscvw/lib/perl';
use Cluster::Cgi;
use Cluster::Common;
use Cluster::Transport;

# Create the html query object
my $q = new Cluster::Cgi;
print $q->header(-expires => "now");

my $transport = new Cluster::Transport;
$transport->get_commandoutput();
my @j_list = $transport->get_junctionlist();

my @c_list = $transport->get_cablelist();

my @n_list = $transport->get_nodelist();

my @a_list;

# Get adapter status info
my %a_map; # Map to adapter index number
for (my $i = 0; $i <= $#n_list; $i++) {
    my @adp_list = $transport->get_adapterlist($n_list[$i]);
    for (my $j = 0; $j <= $#adp_list; $j++) {
	push @a_list, { name => $adp_list[$j]{'name'},
			status => $adp_list[$j]{'status'},
			node => $i};
	$a_map{$n_list[$i].":".$adp_list[$j]{'name'}} = $#a_list;
    }
}

my %j_map; # Reverse map from name to index number
my %n_map;
for (my $i = 0; $i <= $#j_list; $i++) {
    $j_map{$j_list[$i]{'name'}} = $i;
}
for (my $i = 0; $i <= $#n_list; $i++) {
    $n_map{$n_list[$i]} = $i;
}

my @pathstate_list = $transport->get_pathstate_list();

my $length;

# Output adapter info: num_adapters, [node#, name, status]*

$length = @a_list;
print "$length\n";
for (my $j = 0; $j < $length; $j++) {
	print $a_list[$j]{'node'},"\n";
	print $a_list[$j]{'name'},"\n";
	print $a_list[$j]{'status'},"\n";
}

# Output junction info: num_junctions, [ name, status ]*

$length = @j_list;
print "$length\n";
for (my $j = 0; $j < $length; $j++) {
	my $curr_item = $j_list[$j]{'name'};
	my $item_state = $j_list[$j]{'status'};
	print "$curr_item\n";
	print "$item_state\n";
}

# Output cable info: num_cables, then multiple of:
#  cable name
#  cable status
#  endpoint 1 "adapter" or "junction"
#  endpoint 1 name
#  endpoint 1 adapter number (0, 1, ...) or junction number
#  endpoint 2 type
#  endpoint 2 name
#  endpoint 2 adapter/junction

$length = @c_list;
print "$length\n";
for (my $i = 0; $i < $length; $i++) {
	print $c_list[$i]{'status'},"\n";
	for (my $ep = 1; $ep <= 2; $ep++) {
	    my $endpoint = $c_list[$i]{"endpoint$ep"};
	    print lc($c_list[$i]{"type$ep"}),"\n";
	    my $name = $endpoint;
	    if (lc($c_list[$i]{"type$ep"}) eq "adapter") {
		$name =~ s/@.*//; # Remove port
	    } else {
		$name =~ s/@/\@port/; # Add port
	    }
	    print $name,"\n";
	    $endpoint =~ s/@.*//; # Remove port
	    if ($c_list[$i]{"type$ep"} eq "Adapter") {
		print $a_map{$endpoint},"\n";
	    } else {
		print $j_map{$c_list[$i]{"name$ep"}},"\n";
	    }
	}
}

# Print path status: # of faulted paths, then [status, ep1, ep2, cable#...]*
my @path_problems;

# Look at paths through junctions

# Create map from endpoints to status
my %pathstatemap;
foreach my $ps (@pathstate_list) {
    $pathstatemap{$ps->{'endpoint1'}, $ps->{'endpoint2'}} = $ps->{'status'};
    $pathstatemap{$ps->{'endpoint2'}, $ps->{'endpoint1'}} = $ps->{'status'};
}

foreach my $j (@j_list) {
    for (my $c1num = 0; $c1num <= $#c_list; $c1num++) {
	my ($c1) = $c_list[$c1num];
	if (lc($c1->{"type2"}) ne "junction" ||
		$c1->{"name2"} ne $j->{'name'}) {
	    next;
	}
	for (my $c2num = $c1num+1; $c2num <= $#c_list; $c2num++) {
	    my ($c2) = $c_list[$c2num];
	    if (lc($c2->{"type2"}) ne "junction" ||
		    $c2->{"name2"} ne $j->{'name'}) {
		next;
	    }
	    # Have two endpoints connected by junction
	    my $ep1 = $c1->{"node1"}.":".$c1->{"name1"};
	    my $ep2 = $c2->{"node1"}.":".$c2->{"name1"};
	    my $status = $pathstatemap{$ep1, $ep2};
	    my $str;
	    my $ep1num = $a_map{$ep1};
	    my $ep2num = $a_map{$ep2};
	    if (!defined($status)) {
		push @path_problems, "offline $ep1num $ep2num";
	    } elsif (lc($status) eq "waiting") {
		push @path_problems, "waiting $ep1num $ep2num";
	    } elsif (lc($status) ne "online") {
		push @path_problems, "faulted $ep1num $ep2num";
	    }
	}
    }
}

# Find directly connected cables
for (my $c1num = 0; $c1num <= $#c_list; $c1num++) {
    my ($c1) = $c_list[$c1num];
    if (lc($c1->{"type1"}) ne "adapter" ||
	    lc($c1->{"type2"}) ne "adapter") {
	next;
    }
    # Have two endpoints connected by cable
    my $ep1 = $c1->{"node1"}.":".$c1->{"name1"};
    my $ep2 = $c1->{"node2"}.":".$c1->{"name2"};
    my $status = $pathstatemap{$ep1, $ep2};
    my $str;
    my $ep1num = $a_map{$ep1};
    my $ep2num = $a_map{$ep2};
    if (!defined($status)) {
	push @path_problems, "offline $ep1num $ep2num";
    } elsif (lc($status) eq "waiting") {
	push @path_problems, "waiting $ep1num $ep2num";
    } elsif (lc($status) ne "online") {
	push @path_problems, "faulted $ep1num $ep2num";
    }
}

print 1+($#path_problems),"\n";
print join("\n", @path_problems);
